			-- Bouncing Ball Video 
			-- UP1PACK - UP1core package
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;
PACKAGE up1core IS
	COMPONENT sevenseg
		PORT(	input: IN STD_LOGIC_VECTOR(3 DOWNTO 0);
				a,b,c,d,e,f,g : OUT STD_LOGIC);
	END COMPONENT;
	COMPONENT debounce
		PORT(pb, clock_100Hz 					: IN	STD_LOGIC;
	         pb_debounced						: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT onepulse
		PORT(pb_debounced, clock				: IN	STD_LOGIC;
		 	 pb_single_pulse					: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT clk_div
		PORT(clock_25Mhz						: IN	STD_LOGIC;
			clock_1MHz							: OUT	STD_LOGIC;
			clock_100KHz						: OUT	STD_LOGIC;
			clock_10KHz							: OUT	STD_LOGIC;
			clock_1KHz							: OUT	STD_LOGIC;
			clock_100Hz							: OUT	STD_LOGIC;
			clock_10Hz							: OUT	STD_LOGIC;
			clock_1Hz							: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT vga_sync
 		PORT(clock_25Mhz, red, green, blue		: IN	STD_LOGIC;
         	red_out, green_out, blue_out		: OUT 	STD_LOGIC;
			horiz_sync_out, vert_sync_out		: OUT 	STD_LOGIC;
			pixel_row, pixel_column				: OUT STD_LOGIC_VECTOR(10 DOWNTO 0));
	END COMPONENT;
	COMPONENT char_rom
		PORT(	character_address				: IN	STD_LOGIC_VECTOR(5 DOWNTO 0);
			font_row, font_col					: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0);
			rom_mux_output						: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT keyboard
		PORT(keyboard_clk, keyboard_data, clock_25Mhz , 
			reset, read							: IN	STD_LOGIC;
			scan_code							: OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
		--	scan_code1				: OUT	STD_LOGIC_VECTOR(7 DOWNTO 0);
			scan_ready							: OUT	STD_LOGIC);
	END COMPONENT;
	COMPONENT mouse
		PORT( 	clock_25Mhz, reset 				: IN std_logic;
         		mouse_data						: INOUT std_logic;
        		mouse_clk 						: INOUT std_logic;
        		left_button, right_button 		: OUT std_logic;
        		mouse_cursor_row, mouse_cursor_column : OUT std_logic_vector(10 DOWNTO 0));       
		END COMPONENT;
END up1core;


			-- Bouncing Ball Video 
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.STD_LOGIC_ARITH.all;
USE IEEE.STD_LOGIC_SIGNED.all;
LIBRARY work;
USE work.up1core.all;


ENTITY ball IS

Generic(ADDR_WIDTH: integer := 12; DATA_WIDTH: integer := 1);

   PORT(SIGNAL Clock		 										: 	IN std_logic;
     
        SIGNAL Red,Green,Blue 										: 	OUT std_logic;
        SIGNAL Horiz_sync,Vert_sync									: 	OUT std_logic;
		SIGNAL    K_clk, K_data, K_reset							: 	IN std_logic; 
			--Keyboard Output
	    SIGNAL 	 K_ready											: 	OUT std_logic;
		signal a_in, b_in, c_in, d_in, e_in, f_in, g_in				:	OUT std_logic;
        signal la_in, lb_in, lc_in, ld_in, le_in, lf_in, lg_in		:	OUT std_logic);
		
END ball;

architecture behavior of ball is

			-- Video Display Signals   
SIGNAL Red_Data, Green_Data, Blue_Data, vert_sync_int,
		reset, Ball_on,Ball_on1,Ball_on2, Direction					: 	std_logic;
SIGNAL Size, Size1 													: 	std_logic_vector(10 DOWNTO 0);  
SIGNAL Ball_X_motion,Ball_X1_motion ,Ball_Y_motion,Ball_Y1_motion   : 	std_logic_vector(10 DOWNTO 0);
SIGNAL Ball_X2_motion ,Ball_Y2_motion  								: 	std_logic_vector(10 DOWNTO 0);

SIGNAL Ball_Y_pos,Ball_Y1_pos, Ball_X_pos, Ball_X1_pos,Ball_Y2_pos, 
Ball_X2_pos															: 	std_logic_vector(10 DOWNTO 0);

SIGNAL pixel_row, pixel_column										: 	std_logic_vector(10 DOWNTO 0); 
			-- Signals for LED Display
SIGNAL LSB,MSB														: 	std_logic_vector(3 DOWNTO 0);
SIGNAL LSB_7SEG,MSB_7SEG											: 	std_logic_vector(6 DOWNTO 0);
SIGNAL TEST,TEST1,TEST2												:	std_logic;
SIGNAL con															:	std_logic := '0'; 
SIGNAL move,move1													: 	std_logic;
SIGNAL scan_code_buf,scan_code_buf1									: 	STD_LOGIC_VECTOR(7 downto 0);
SIGNAL fast 														: 	STD_LOGIC_VECTOR(1 DOWNTO 0) := "00";
SIGNAL K_rdy														: 	std_logic;
SIGNAL K_read														: 	std_logic;
SIGNAL const_char 													: 	STD_LOGIC_VECTOR(5 DOWNTO 0) := "100000" ;
SIGNAL char_address													: 	std_logic_vector(5 DOWNTO 0);
SIGNAL rom_mux_output												: 	std_logic;
SIGNAL col_address, row_address										: 	std_logic_vector(5 DOWNTO 0);
SIGNAL speed 														: 	STD_LOGIC_VECTOR(1 DOWNTO 0) := "00";

SIGNAL ball_row														: 	std_logic_vector(4 DOWNTO 0) := "01010";
SIGNAL ball_col														: 	std_logic_vector(5 DOWNTO 0) := "010100";
SIGNAL R_sw,G_sw,B_sw 												: 	STD_LOGIC := '0';
signal number, lnumber												: 	std_logic_vector(3 downto 0);
signal comp															:  	std_logic_vector(5 downto 0) := "110000";
signal player:std_logic_vector(5 downto 0):= "110000";
BEGIN           
   SYNC: vga_sync
 		PORT MAP(clock_25Mhz => clock, 
				red => red_data, green => green_data, blue => blue_data,	
    	     	red_out => red, green_out => green, blue_out => blue,
			 	horiz_sync_out => horiz_sync, vert_sync_out => vert_sync_int,
			 	pixel_row => pixel_row, pixel_column => pixel_column);
			 	
		Keys_b: keyboard
		PORT MAP(	 keyboard_clk => K_clk,
				     keyboard_data => K_data,
				     clock_25Mhz => clock,
				     reset => K_reset,
				     read =>  K_read,
				
				--Outputs
				    scan_code => scan_code_buf,
				   -- scan_code1 => scan_code_buf1,
				    scan_ready => K_rdy
				    );	 
	   CGROM: CHAR_ROM
		PORT MAP(character_address => char_address,
				font_row => pixel_row(3 DOWNTO 1), font_col => pixel_column(3 DOWNTO 1),	
				rom_mux_output => rom_mux_output);
							 
		sev_seg: sevenseg
			port map(	input => number,
						a => a_in,
						b => b_in,
						c => c_in,
						d => d_in,
						e => e_in,
						f => f_in,
						g => g_in);
			lsev_seg: sevenseg
			port map(	input =>lnumber,
						a => la_in,
						b => lb_in,
						c => lc_in,
						d => ld_in,
						e => le_in,
						f => lf_in,
						g => lg_in);
				


Size  <= CONV_STD_LOGIC_VECTOR(5,11);
Size1 <= CONV_STD_LOGIC_VECTOR(26,11);

Ball_X1_pos <= CONV_STD_LOGIC_VECTOR(640,11) - CONV_STD_LOGIC_VECTOR(5,11);
Ball_X2_pos <= CONV_STD_LOGIC_VECTOR(5,11);

-- To display Char
row_address(5 DOWNTO 0) <= pixel_row(9 DOWNTO 4);
col_address(5 DOWNTO 0) <= pixel_column(9 DOWNTO 4);

		-- need internal copy of vert_sync to read
vert_sync <= vert_sync_int;

Green_Data <=   Not( rom_mux_output or Ball_on or B_sw);
Red_Data <=    (rom_mux_output  or Ball_on1 or Ball_on2 or G_sw );
Blue_Data <=  NOT(rom_mux_output or R_sw);

 
	
MSB <= Ball_Y_pos(7 DOWNTO 4);
LSB <= Ball_Y_pos(3 DOWNTO 0);


RGB_Display: Process (Ball_X_pos, Ball_Y_pos, Ball_Y1_pos,Ball_X1_pos, pixel_column, pixel_row, Size, Size1)
BEGIN
			-- Set Ball_on ='1' to display ball
 IF ('0' & Ball_X_pos <= pixel_column + Size) AND
 			-- compare positive numbers only
 	(Ball_X_pos + Size >= '0' & pixel_column) AND
 	('0' & Ball_Y_pos <= pixel_row + Size) AND
 	(Ball_Y_pos + Size >= '0' & pixel_row ) THEN

 		Ball_on <= '1';
 	ELSE
 		Ball_on <= '0';
END IF;


 IF ('0' & Ball_X1_pos <= pixel_column + Size) AND
 			-- compare positive numbers only
 	(Ball_X1_pos + Size >= '0' & pixel_column) AND
 	('0' & Ball_Y1_pos <= pixel_row + Size1) AND
 	(Ball_Y1_pos + Size1 >= '0' & pixel_row ) THEN

 		Ball_on1<= '1';
 	ELSE
 		Ball_on1 <= '0';
END IF;

IF ('0' & Ball_X2_pos <= pixel_column + Size) AND
 			-- compare positive numbers only
 	(Ball_X2_pos + Size >= '0' & pixel_column) AND
 	('0' & Ball_Y2_pos <= pixel_row + Size1) AND
 	(Ball_Y2_pos + Size1 >= '0' & pixel_row ) THEN

 		Ball_on2<= '1';
 	ELSE
 		Ball_on2 <= '0';
END IF;
END process RGB_Display;
--
 
VIDEO_DISPLAY_DATA: PROCESS

variable cnt: integer range 0 to 25175000;
variable speed1: integer range 0 to 1000343 := 1000340;

BEGIN
  WAIT UNTIL (clock'event) AND (clock='1');
	
		CASE speed IS
			WHEN "00" =>
			speed1 := 1000340;
			WHEN "01" =>
			speed1 := 1000340/2;
			WHEN "10" =>
			speed1 := 1000340/4;
			WHEN OTHERS =>
			speed1 := 1000340/8;
		END CASE;	
	

--GET KEYBOARD CHARACTER	
IF(cnt = speed1+1)THEN
	IF(K_rdy = '1')THEN
		
		
		case scan_code_buf is
			WHEN "01110101" =>    
				const_char <= "011110";
			WHEN "01101011" =>   
				const_char <= "011111";
				--direction_horz <= '1';
			WHEN "01110100" => 
				const_char <= "011101";
			WHEN "01101001" =>
				R_sw <= not(R_sw);
			WHEN "01110000" =>
				G_sw <= not(G_sw);
			WHEN "00000100" =>
				B_sw <= not(G_sw);		
 			WHEN "01110010" => --R
  		     const_char <= "011100";	
	
																						
			WHEN OTHERS	=>
				const_char <= "100000";
		END CASE;
		
		K_read <= '1';		
	ELSE
		K_read <= '0';
	END IF;	
END IF;	
	

  IF(cnt = speed1+1) THEN
		cnt := 0;
  ELSE
		cnt := cnt + 1;
END IF;


								
IF (row_address = "00010") AND (col_address = "001111") THEN
			char_address <= "010000";
ELSIF (row_address = "00010") AND (col_address = "010000") THEN
			char_address <= "001001";
ELSIF (row_address = "00010") AND (col_address = "010001") THEN
			char_address <= "001110";
ELSIF (row_address = "00010") AND (col_address = "010010") THEN
		char_address <= "000111";
ELSIF (row_address = "00010") AND (col_address = "010011") THEN
		 char_address <= "100000"; 
ELSIF (row_address = "00010") AND (col_address = "010100") THEN
		char_address <= "010000";					
ELSIF (row_address = "00010") AND (col_address = "010101") THEN
		char_address <= "001111";			
ELSIF (row_address = "00010") AND (col_address = "010110") THEN
		char_address <= "001110";
ELSIF (row_address = "00010") AND (col_address = "010111") THEN
	  char_address <= "000111";			  		
ELSIF (row_address = "00010") AND (col_address = "011001") THEN
		char_address <= const_char;

--Showing characters in first line "PLAYER"		
ELSIF (row_address = "00001") AND (col_address = "100000") THEN
		char_address <= "010000";		
ELSIF (row_address = "00001") AND (col_address = "100001") THEN
		char_address <= "001100";
ELSIF (row_address = "00001") AND (col_address = "100010") THEN
		char_address <= "000001";
ELSIF (row_address = "00001") AND (col_address = "100011") THEN
		char_address <= "011001";
ELSIF (row_address = "00001") AND (col_address = "100100") THEN
		char_address <= "000101";	
ELSIF (row_address = "00001") AND (col_address = "100101") THEN
		char_address <= "010010";	
		
--showing characters in first coloum "comp"
ELSIF (row_address = "00001") AND (col_address = "000011") THEN
		char_address <= "000011";		
ELSIF (row_address = "00001") AND (col_address = "000100") THEN
		char_address <= "001111";
ELSIF (row_address = "00001") AND (col_address = "000101") THEN
		char_address <= "001101";
ELSIF (row_address = "00001") AND (col_address = "000110") THEN
		char_address <= "010000";

		-- This displays the score for computer
ELSIF (row_address = "00010") AND (col_address = "000101") THEN
					char_address <= comp;     
ELSIF (row_address = "00010") AND (col_address = "000100") THEN
	   
					char_address <= "110000";
	   -- This displays the score for player
ELSIF (row_address = "00010") AND (col_address = "100010") THEN
					char_address <= "110000";	
ELSIF (row_address = "00010") AND (col_address = "100011") THEN
					char_address <= player;		
ELSIF (row_address = "00011") THEN
		char_address <= "101101";
ELSE
		char_address <= "100000";
END IF;

END PROCESS VIDEO_DISPLAY_DATA;



Move_Ball: process
variable fast : integer range 0 to 50 := 1; 

BEGIN
			-- Move ball once every vertical sync
	WAIT UNTIL vert_sync_int'event and vert_sync_int = '1';
			-- Bounce off top or bottom of screen		
			
			      
			    IF ('0' & Ball_Y2_pos) >= CONV_STD_LOGIC_VECTOR(480,11) - Size THEN
			    Ball_Y2_motion <= - CONV_STD_LOGIC_VECTOR(3,11);
			    ELSIF Ball_Y2_pos <= CONV_STD_LOGIC_VECTOR(80,11)  THEN
				Ball_Y2_motion <= CONV_STD_LOGIC_VECTOR(3,11);
			    END IF;
			    -- Compute next ball Y position
			    Ball_Y2_pos <= Ball_Y2_pos + Ball_Y2_motion;
			      
			  
			  
		        case scan_code_buf is
			    WHEN "01110101" =>
					Ball_Y1_motion <= - CONV_STD_LOGIC_VECTOR(3,11);
					Ball_Y1_pos <= Ball_Y1_pos + Ball_Y1_motion;
				
		            IF Ball_Y1_pos <= Size1 + CONV_STD_LOGIC_VECTOR(60,11)THEN
		            Ball_Y1_pos <=  Size1 + CONV_STD_LOGIC_VECTOR(60,11); 
		            scan_code_buf1 <= "00000000"; 
		            end if; 
		     
			    WHEN  "01110010"    =>
					Ball_Y1_motion <= CONV_STD_LOGIC_VECTOR(3,11);
					Ball_Y1_pos <= Ball_Y1_pos + Ball_Y1_motion;
			 
					IF ('0' & Ball_Y1_pos) >= CONV_STD_LOGIC_VECTOR(480,11) - Size1 THEN
		            Ball_Y1_pos <= CONV_STD_LOGIC_VECTOR(480,11) - Size1;
		            end if;
					scan_code_buf1 <= "00000000"; 
		        WHEN  "01101011"    =>   
					fast := fast + 1; 
					if( fast >= 5 ) then
					fast := 5; 
					end if;
		        WHEN  "01110100"    =>   
					fast := fast - 1; 
					if( fast <= 1 ) then
					fast := 1; 
					end if; 
			    WHEN OTHERS	   =>
			
					Ball_Y_pos  <= CONV_STD_LOGIC_VECTOR(240,11);
		        END CASE;
		
			 
			 
		
				
				-- MOVING AND BONSIG BALL 			
			IF ((Ball_Y_pos) <= (Ball_Y1_pos + CONV_STD_LOGIC_VECTOR(26,11))) and (Ball_Y_pos )>= (Ball_Y1_pos - CONV_STD_LOGIC_VECTOR(26,11)) THEN
            TEST1 <='0';
            ELSE  
            TEST1 <='1';
            END IF; 
				
			
			IF ('0' & Ball_Y_pos) >= CONV_STD_LOGIC_VECTOR(480,11) - Size THEN
			Ball_Y_motion <= - CONV_STD_LOGIC_VECTOR(3,11);
			ELSIF Ball_Y_pos <= CONV_STD_LOGIC_VECTOR(80,11)THEN
			Ball_Y_motion <= CONV_STD_LOGIC_VECTOR(3,11);
			END IF;
			   

		    IF (Ball_X_pos >= CONV_STD_LOGIC_VECTOR(640,11) - (Size + Size+ Size + CONV_STD_LOGIC_VECTOR(6,11))) THEN 
               TEST2 <= '0'; 
            ELSIF (Ball_X_pos <=  (Size + Size+ Size + CONV_STD_LOGIC_VECTOR(6,11)))  THEN 
               TEST2 <= '1'; 
            END IF; 
				
            IF  TEST2 = '0' and con = '1'THEN
		       TEST <= '1';
		    ELSIF TEST2 = '1' and con = '0'THEN
		       TEST  <= '0';
		    END IF; 
			 
            IF TEST = '1' THEN 
                Ball_X_motion <= - CONV_STD_LOGIC_VECTOR(2,11);
                Ball_X_pos <= Ball_X_pos + Ball_X_motion;
			if  Ball_X_pos <= (Size + Size+ Size + CONV_STD_LOGIC_VECTOR(6,11))THEN
                 con <= '0';
            end if; 
               
            ELSIF TEST = '0' THEN 
				Ball_X_motion <= CONV_STD_LOGIC_VECTOR(2,11);
				Ball_X_pos <= Ball_X_pos + Ball_X_motion;
      
            IF ((Ball_X_pos) = (Ball_X1_pos - (Size + Size+ Size + CONV_STD_LOGIC_VECTOR(6,11))))and TEST1 ='0'  THEN
		          
		         con <= '1';  --Ball hits the paddle
		         if (lnumber = "1001" ) then
					lnumber <= "0000";
					number <= "0000";
					player <= "110000";
					comp <= "110000";
				 else
					lnumber <= lnumber + 1;  --player
					player <= player + "000001";
					
					
				 end if;
					
		      ELSIF ((Ball_X_pos) = (Ball_X1_pos - (Size + Size+ Size + CONV_STD_LOGIC_VECTOR(6,11))))and TEST1 ='1'  THEN
		           
		            Ball_X_pos <= CONV_STD_LOGIC_VECTOR(320,11);
		            if (number = "1001") then
		            number <= "0000";
		            lnumber <= "0000";
		            comp <= "110000";
		            player <= "110000";
		            else
		            number <= number + 1;	
		            comp <= comp + "000001";	--computer
		            end if;
		           
		          END IF; 
               END IF;
			Ball_Y_pos <= Ball_Y_pos + Ball_Y_motion;
END process Move_Ball;

END behavior;

